var {Cc, Ci, Cu} = require("chrome");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.defineLazyGetter(this, "UnifiedUrlbar", () => {
  Cu.unload("chrome://unified-urlbar/content/UnifiedUrlbar.jsm");
  let scope = {};
  Cu.import("chrome://unified-urlbar/content/UnifiedUrlbar.jsm", scope);
  return scope.UnifiedUrlbar;
});
const prefSvc = require('sdk/preferences/service');

var gUninstalled = false;
try {
  // This module only exists if the opt-out study infrastructure is present.
  Cu.import("resource://shield-recipe-client/lib/AddonStudies.jsm", this);
} catch (ex) {
  this.AddonStudies = null;
}

const feature = {
  isEligible() {
    return UnifiedUrlbar.isUserEligible();
  },

  checkHasConflicts() {
    // Exclude users having the Activity Stream add-on.
    const { AddonManager } = Cu.import("resource://gre/modules/AddonManager.jsm");
    return new Promise(resolve => {
      AddonManager.getAddonsByIDs(["@activity-streams", "universal-search@mozilla.com"],
        results => resolve(results.some(r => !!r)));
    });
  },

  enableExtendedTelemetry() {
    // Enable extended telemetry if needed, but note this requires a restart
    // before we can start collecting data.
    if (!prefSvc.isSet("browser.urlbar.experiment.unified-urlbar.telemetry.enabled.mirror")) {
      let telemetryEnabled = prefSvc.get("toolkit.telemetry.enabled", false);
      if (!telemetryEnabled) {
        prefSvc.set("browser.urlbar.experiment.unified-urlbar.telemetry.enabled.mirror", telemetryEnabled);
        prefSvc.set("toolkit.telemetry.enabled", true);
      }
    }
  },

  install(variation, config) {
    if (!AddonStudies) {
      config.installMode = "opt-in";
      this.enableExtendedTelemetry();
    } else {
      try {
        AddonStudies.getAll().then(studies => {
          // Consider a possible asynchronous execution after uninstalling.
          if (gUninstalled) {
            return;
          }
          config.installMode = "opt-out";
        }).catch(Cu.reportError);
      } catch(ex) {
        Cu.reportError(ex);
      }
    }

    if (config.phasedVariation && !config.phasedExpired) {
      variation = "control";
    }

    // Start the experiment only on version 56.
    let vc = Cc["@mozilla.org/xpcom/version-comparator;1"]
               .getService(Ci.nsIVersionComparator);
    let appInfo = Cc["@mozilla.org/xre/app-info;1"]
                    .getService(Ci.nsIXULAppInfo);
    if (vc.compare(appInfo.version, "56") < 0) {
      variation = "control";
    }

    if (variation == "control") {
      UnifiedUrlbar.init({
        removeSearchbar: false,
        resultsLimit: false,
        matchBuckets: "",
        matchBucketsSearch: "",
      }, prefSvc);
      return;
    }

    let variationMatrix = {
      "minimal-top-history":  ["general:1,suggestion:3,general:2"],
      "minimal-top-same":     ["general:1,suggestion:3,general:2",
                               "suggestion:1,general:2,suggestion:3"],
      "minimal-top-opposite": ["suggestion:1,general:2,suggestion:3",
                               "general:1,suggestion:3,general:2"],

      "minimal-history":      ["general:3,suggestion:3"],
      "minimal-search":       ["suggestion:3,general:3"],
      "minimal-same":         ["general:3,suggestion:3",
                               "suggestion:3,general:3"],

      "minimal-mosthistory":  ["general:6"],
      "minimal-mostsearch":   ["suggestion:6"],

      "unified-top-history":  ["general:1,suggestion:3,general:5"],

      "unified-history":      ["general:5,suggestion:4"],
      "unified-search":       ["suggestion:4,general:5"],
    };

    UnifiedUrlbar.init({
      removeSearchbar: true,
      resultsLimit: variation.includes("minimal") ? 7 : 10,
      matchBuckets: variationMatrix[variation][0],
      matchBucketsSearch: variationMatrix[variation][1],
    }, prefSvc);
  },

  cleanup() {
    UnifiedUrlbar.destroy(prefSvc);
  },

  diagnostics() {
    return UnifiedUrlbar.diagnostics();
  },

  uninstall() {
    gUninstalled = true;
    if (prefSvc.isSet("browser.urlbar.experiment.unified-urlbar.telemetry.enabled.mirror")) {
      let telemetryEnabled = prefSvc.get("browser.urlbar.experiment.unified-urlbar.telemetry.enabled.mirror", false);
      prefSvc.reset("browser.urlbar.experiment.unified-urlbar.telemetry.enabled.mirror");
      prefSvc.set("toolkit.telemetry.enabled", telemetryEnabled);
    }
  }
};

exports.feature = feature;
