const self = require("sdk/self");
const shield = require("shield-studies-addon-utils");
const { when: unload } = require("sdk/system/unload");
const { feature } = require('./feature');
const prefSvc = require('sdk/preferences/service');

const studyConfig = {
  name: self.id,
  duration: 14,
  variations: {},
  surveyUrls:  {
    'end-of-study': "https://qsurvey.mozilla.com/s3/search-composition-1",
    'user-ended-study': "https://qsurvey.mozilla.com/s3/search-composition-1",
    'ineligible': null
  },
}

for (let variation of [
  "minimal-top-history",
  "minimal-top-same",
  "minimal-top-opposite",
  "minimal-history",
  "minimal-search",
  "minimal-same",
  "minimal-mosthistory",
  "minimal-mostsearch",
  "unified-top-history",
  "unified-history",
  "unified-search",
  "control",
]) {
  let variationName = variation;
  studyConfig.variations[variation] =
    config => feature.install(variationName, config);  
}

class OurStudy extends shield.Study {
  constructor(config) {
    super(config);
  }

  isEligible() {
    return super.isEligible() && feature.isEligible();
  }

  whenIneligible () {
    // Additional actions when the user isn't eligible.
    super.whenIneligible();
  }

  whenInstalled() {
    // Additional actions when the study gets installed.
    super.whenInstalled();
  }

  cleanup() {
    super.cleanup();
    feature.cleanup();
  }

  diagnostics() {
    return feature.diagnostics();
  }

  whenComplete () {
    // Additional actions when the study is naturally completed.
    super.whenComplete();  // calls survey, uninstalls
  }

  whenUninstalled () {
    // Additional actions when the user uninstalls the study.
    super.whenUninstalled();
    feature.uninstall();
  }

  decideVariation () {
    return super.decideVariation(); // chooses at random
  }
}

const thisStudy = new OurStudy(studyConfig);
unload((reason) => thisStudy.shutdown(reason))

exports.study = thisStudy;
exports.checkHasConflicts = feature.checkHasConflicts;
